-- --------------------------------------------------------------------------
-- Configure the GPIO-0 pin to output the 32KHz Clock signal
-- --------------------------------------------------------------------------




-- --------------------------------------------------------------------------
-- This is an example program written in Euphoria.
-- Euphoria is a High-Level interpreted language, that is easy to understand
-- and provides a quick means of writing programs for testing and debugging 
-- hardware specific issues.
--
-- Note:  Two dashes (--) are used to start a comment
--        HEX numbers are prefixed with the pound sign (#)
-- --------------------------------------------------------------------------



-- --------------------------------------------------------------------------
-- Library Routines to Include
-- --------------------------------------------------------------------------
include ports32.e                       -- needed for 32-bit I/O Port access



-- --------------------------------------------------------------------------
-- Global Constants and Variables
-- --------------------------------------------------------------------------
global constant SbPciConfig = #80009000 -- This is the base address 
                                        -- for all South Bridge
                                        -- PCI Configuration Index Registers.
    
global constant PciIndexPort = #CF8     -- PCI Configuration Index Register
global constant PciDataPort = #CFC      -- PCI Configuration Data Register
                                




-- --------------------------------------------------------------------------
-- This function reads and returns a 32-bit data item from the 
-- South Bridge - PCI Configuration Registers
-- --------------------------------------------------------------------------
function SB_PCI_READ(word FunctionNumber, word Index)

    atom Offset                         -- Euphoria type "atom" is required
                                        -- to hold 32-bit unsigned integers.

    Offset = SbPciConfig                -- Start with the Base Address.
           + (FunctionNumber * 256)     -- The arguments "FunctionNumber"
           + Index                      -- and "Index" which are passed into 
                                        -- this routine, are used to arrive 
                                        -- at the actual value used.

    outp32(PciIndexPort, Offset)        -- send calculated Index to Port 
    return inp32(PciDataPort)           -- return the desired Data

end function


-- --------------------------------------------------------------------------
-- Configure the internal multiplexer to route the 32KHz clock signal to
-- the GPIO-0 pin
-- --------------------------------------------------------------------------
atom F3Bar0                             -- PCI Function #3 Base Address Reg 0
atom IoCtrl1                            -- XBus I/O Control Register #1

F3Bar0 = SB_PCI_READ(3, #10)            -- get current F3 Base Address
F3Bar0 = and_bits(F3Bar0, #0000FFFC)    -- clear 2 Least Significant Bits

IoCtrl1 = inp32(F3Bar0)                 -- get current value
IoCtrl1 = or_bits(IoCtrl1, #00400000)   -- route 32KHz clock to GPIO-0
outp32(F3Bar0, IoCtrl1)                 -- enable changes



-- --------------------------------------------------------------------------
-- Configure the GPIO-0 pin as an output
-- --------------------------------------------------------------------------
atom F0Bar0                             -- PCI Function #0 Base Address Reg 0
word IntEnable0                         -- GPIO Interrupt Enable 0 Register
word PinCfgSelReg                       -- GPIO Pin Configuration Select Reg
word PinCfgAccReg                       -- GPIO Pin Configuration Access Reg
atom CfgData                            -- current Pin Configuration Data

F0Bar0 = SB_PCI_READ(0, #10)            -- get current F0 Base Address #0
F0Bar0 = and_bits(F0Bar0, #0000FFFC)    -- clear 2 Least Significant Bits

IntEnable0 = F0Bar0 + #8                -- GPIO Interrupt Enable 0 Register
outp32(IntEnable0, 0)                   -- dissable PME generation (all GPIOs)

PinCfgSelReg = F0Bar0 + #20             -- GPIO Pin Configuration Select Reg
outp32(PinCfgSelReg, 0)                 -- Select GPIO-0

PinCfgAccReg = F0Bar0 + #24             -- GPIO Pin Configuration Access Reg
CfgData = inp32(PinCfgAccReg)           -- get current value
CfgData = and_bits(CfgData, #FFFFFFF8)  -- clear PME & Output related bits
CfgData = or_bits(CfgData, 3)           -- configure as Push/Pull Output
outp32(PinCfgAccReg, CfgData)           -- enable pin


